// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// main hdclient application window class

// #define _DEBUG

#include <qapplication.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

extern "C"
{
#include "lame.h"
}

#include "mp3stream.h"


#include "hdclient.h"
#include "hdclient_basis.h"

//===========================================================================
// constructor for the client program
//===========================================================================

HdClient::HdClient(QWidget *parent,
               QApplication *mainApplication)
{

   m_BasicData = NULL;



   // check, whether we can access the hardware
   m_qs_SoundDevice = "/dev/dsp";
   m_qs_MixerDevice = "/dev/mixer";

   m_i_OutputLevel       = 50;
   m_i_OutputLevelMaster = 50;

   m_i_IPPort = DEFAULT_SERVER_PORT;
   m_qs_IPAddress = "localhost";


   // allocate a saveblockarray
   pc_SaveBlocks = new SaveBlockArray();

   m_pc_MainDialog = NULL;


   m_ProgramState.b_PlayStarted = false;
   m_ProgramState.b_GlobalStop  = false;
   if(parent == 0)
   {
      m_ProgramState.b_CommandLineMode = true;
   }
   else
   {
      m_ProgramState.b_CommandLineMode = false;
   }


   if( ! m_ProgramState.b_CommandLineMode)
   {
      // create the basic data
      m_BasicData = new BasisDatenType();

      // read basisdata file
      BasicDataRead(this);
   }

   m_i_MixerOutputDevice = SOUND_MIXER_LINE;	
   m_pc_Mixer = new MixerDevice(m_qs_MixerDevice);

   // generate the audio player
   pc_AudioPlayer = new AudioPlayer(m_qs_SoundDevice);
   pc_AudioPlayer->SetSpeed(44100);


} // HdClient()

//===========================================================================
// update the time display
//===========================================================================

HdClient::~HdClient()
{
   
} // destructor

//===========================================================================
//
//===========================================================================

void * HdClient::SocketThreadCB(void * p_Pointer)
{
   HdClient * pc_Client;
   pc_Client = (HdClient *) p_Pointer;

   return pc_Client->SocketThread();

} // SocketThreadCB()

//===========================================================================
//
//===========================================================================

void * HdClient::SocketThread()
{
   int i_BindResult;
   int i_ConnectResult;
   int i_RecvResult;
   int i_BufferFilled;

   lame_decode_init();

   ClientSocket = socket(PF_INET,SOCK_STREAM,0);
   if(ClientSocket == -1)
   {
      fprintf(stderr,"Invalid Server Socket\n");
   }

   ClientAddress.sin_family      = PF_INET;
   ClientAddress.sin_addr.s_addr = INADDR_ANY;
   ClientAddress.sin_port        = 0;

   i_BindResult = bind(ClientSocket,
                       (struct sockaddr *) &ClientAddress,
                       sizeof(ClientAddress));


   if(i_BindResult !=0)
   {
      fprintf(stderr,"Error: BindResult = %d\n",i_BindResult);
      return NULL;
   }

   ServerAddress.sin_family      = PF_INET;
   ServerAddress.sin_addr.s_addr = GetAddress(m_qs_IPAddress);
   ServerAddress.sin_port        = m_i_IPPort;


   do
   {
      i_ConnectResult = connect(ClientSocket,
                              (struct sockaddr *) &ServerAddress,
                              sizeof(ServerAddress)
                            );
      if(i_ConnectResult <0)
      {
         sleep(1);
      }
   } while(i_ConnectResult <0);

   if(  m_ProgramState.b_CommandLineMode)
   {
      fprintf(stdout,"Connected to %s\n",(const char *) m_qs_IPAddress);
   }
   // fprintf(stderr,"Connect Result = %d\n",i_ConnectResult);
   if(m_p_MainDialog != NULL)
   {
      m_p_MainDialog->SetStatusLabel("Connected");
   }

   while(true)
   {
      // test for stop mode
      if(m_ProgramState.b_GlobalStop)
      {
         CloseConnection();
         return NULL;
      }

      // try to get a full mp3 stream buffer
      {
         Mp3StreamBuffer *pc_StreamBuffer;

         i_BufferFilled =0;

         while (true)
         {
            if(m_ProgramState.b_GlobalStop)
            {
               CloseConnection();
               return NULL;
            }

            i_RecvResult =    recv(ClientSocket,
                             &uc_ReceiveBuffer[i_BufferFilled],
                             sizeof(uc_ReceiveBuffer) - i_BufferFilled,
                             0);

            if(i_RecvResult >0)
            {
               i_BufferFilled += i_RecvResult;
               pc_StreamBuffer = (Mp3StreamBuffer *) uc_ReceiveBuffer;
               if(pc_StreamBuffer->i_TransferSize == i_BufferFilled)
               {
                  break;
               }
            } // if block has data
         } // while not full


         // test for stop mode
         if(m_ProgramState.b_GlobalStop)
         {
            CloseConnection();
            return NULL;
         }
      } // block for a complete buffer


      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->TurnGraphics();
      }

      if(i_BufferFilled != 0)
      {
         SaveBlock * pc_ActualSaveBlock;
         int i_DecodeResult;

         Mp3StreamBuffer *pc_StreamBuffer;
         // decode the buffer and queue it into the
         // fprintf(stderr,"Recv %d \n",i_RecvResult);

         pc_StreamBuffer = (Mp3StreamBuffer *) uc_ReceiveBuffer;

         i_DecodeResult = lame_decode((char *) pc_StreamBuffer->uc_Mp3Buffer,
                                       pc_StreamBuffer->i_BufferFilled,
                                       us_LeftPCM,
                                       us_RightPCM
                     );
         //fprintf(stderr,"decoded of %d  = %d\n",
         //             pc_StreamBuffer->i_BufferFilled,
         //             i_DecodeResult);

         // test for stop mode
         if(m_ProgramState.b_GlobalStop)
         {
            CloseConnection();
            return NULL;
         }

         if(i_DecodeResult >0)
         {
            int i;
            unsigned short us_Left;
            unsigned short us_Right;
            unsigned char  *puc_Combined;
            // put the result into a saveblock
            pc_ActualSaveBlock = pc_SaveBlocks->GetNextRecordBlock();
            // copy it into the buffer

            pc_ActualSaveBlock->BufferFilled = i_DecodeResult*4;
            puc_Combined = pc_ActualSaveBlock->Buffer;
            for(i=0;i<i_DecodeResult;i++)
            {
               us_Left  = us_LeftPCM[i];
               us_Right = us_RightPCM[i];
               *puc_Combined++ = (unsigned char) (us_Left & 0xff );
               *puc_Combined++ = (unsigned char) ((us_Left>>8) & 0xff );
               *puc_Combined++ = (unsigned char) (us_Right & 0xff );
               *puc_Combined++ = (unsigned char) ((us_Right>>8) & 0xff );

            } // for i
            pc_SaveBlocks->ReleaseRecordBlock();

         } // if decoding successfull

      }
   }

   return NULL;

} // SocketThread()


//===========================================================================
//
//===========================================================================

void * HdClient::PlayerThreadCB(void * p_Pointer)
{
   HdClient * pc_Client;
   pc_Client = (HdClient *) p_Pointer;

   return pc_Client->PlayerThread();

} // PlayerThreadCB()


//===========================================================================
//
//===========================================================================




void * HdClient::PlayerThread()
{
   bool b_FirstBlock;
   SaveBlock *pc_ActualBlock;
   int i_PlayResult ; 


   b_FirstBlock = true;

   while(true)
   {
      // test for stop mode
      if(m_ProgramState.b_GlobalStop)
      {
         // fprintf(stderr,"Stop player thread\n");
         return NULL;
      }         

       pc_ActualBlock = pc_SaveBlocks->GetNextWriteBlock();
       if(b_FirstBlock)
       {
          // wait some seconds;

          time_t t_ActualTime;
          b_FirstBlock = false;



         time( & t_ActualTime);     
     
   
         // wait 3 seconds
         t_PlayerTimespec.tv_sec  =  t_ActualTime+3;
         t_PlayerTimespec.tv_nsec =  1;

         
         pthread_cond_timedwait( &p_PlayerCondition,
                              &p_PlayerMutex,
                              &t_PlayerTimespec);

       } // if first block


      // fprintf(stderr,"Play a noise\n");
      // now do the noise
      i_PlayResult = pc_AudioPlayer->PlayAudio(pc_ActualBlock->Buffer,
                                pc_ActualBlock->BufferFilled);      

      // fprintf(stderr,"result = %d\n",i_PlayResult);
       
   } // while true

} // PlayerThread()

//===========================================================================
//
//===========================================================================


void  HdClient::Beenden()

{
   // if the player is active, stop it before
   DoStop();

   // dont write basedata in command line mode
   if( ! m_ProgramState.b_CommandLineMode)
   {
      // before exiting write the basis data
      if(m_BasicData != NULL)
      {
         BasicDataWrite(this);
      }
   }


   exit(0);
}


//===========================================================================
//
//===========================================================================

void  HdClient::DoStart()

{
   if( ! m_ProgramState.b_PlayStarted )
   {
      m_ProgramState.b_GlobalStop  = false;

      pc_SaveBlocks -> Initialize();

      // start the socket thread
      // init thread attributes
      pthread_attr_init(&SocketAttribut);
      pthread_create(&p_SocketThread,
                     &SocketAttribut,
                     HdClient::SocketThreadCB,
                     this);
   
      pthread_attr_init(&PlayerAttribut);
      pthread_create(&p_PlayerThread,
                     &PlayerAttribut,
                     HdClient::PlayerThreadCB,
                     this);

      m_ProgramState.b_PlayStarted = true;

      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetSensitivity(m_ProgramState);
      }

   }   
} // DoStart()


//===========================================================================
//
//===========================================================================
void  HdClient::DoStop()
{
   if(m_ProgramState.b_PlayStarted)
   {
      // signal the threads the termination
      m_ProgramState.b_GlobalStop  = true;
      // wait for termination
      pthread_join(p_SocketThread,NULL);
      pthread_join(p_PlayerThread,NULL);


      m_ProgramState.b_PlayStarted = false;

      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetStatusLabel("Disconnected");
         m_p_MainDialog->SetSensitivity(m_ProgramState);
      }
   }
} // DoStop()


//===========================================================================
//
//===========================================================================

void  HdClient::DoSliderValueChanged(int i_Value)
{
   m_i_OutputLevel = i_Value;

   m_pc_Mixer->SetOutputMixer(m_i_MixerOutputDevice,
			      m_i_OutputLevel,
                              m_i_OutputLevelMaster);
} // DoSliderValueChanged()

//===========================================================================
//
//===========================================================================

void  HdClient::DoMasterSliderValueChanged(int i_Value)
{
   m_i_OutputLevelMaster = i_Value;

   m_pc_Mixer->SetOutputMixer(m_i_MixerOutputDevice,
			      m_i_OutputLevel,
                              m_i_OutputLevelMaster);
} // DoMasterSliderValueChanged()
//===========================================================================
//
//===========================================================================

void HdClient::SetMainDialog(MainDialog *pc_MainDialog)
{
   m_p_MainDialog = pc_MainDialog;
   if(m_p_MainDialog != NULL)
   {
      InitializeDisplay();
   }
} // SetMainDialog()

//===========================================================================
// transfer all settings initital to the display
//===========================================================================

void HdClient::InitializeDisplay()
{
   if( m_p_MainDialog == NULL)
   {
      return;
   }
   
   m_p_MainDialog->SetOutputLevel(m_i_OutputLevel);
   m_p_MainDialog->SetOutputLevelMaster(m_i_OutputLevelMaster);

   m_p_MainDialog->SetSensitivity(m_ProgramState);

} // InitializeDisplay()

//===========================================================================
//
//===========================================================================

void HdClient::SetOutputMixer()
{
   // mixer available ?
   if(m_pc_Mixer != NULL)
   {
      // set the valid device
      m_pc_Mixer-> SetMixerDevice(m_qs_MixerDevice);

      // call the mixerdevice object
      m_pc_Mixer-> SetOutputMixer(m_i_MixerOutputDevice,
                                m_i_OutputLevel,
                                m_i_OutputLevelMaster);    
   }
   
} // SetOutputMixer()

//===========================================================================
//
//===========================================================================


in_addr_t HdClient::GetAddress (QString qs_ComputerName)
{
   in_addr_t s_Address;

   // try to get the address from dotted numbers?
   s_Address = inet_addr(qs_ComputerName);

   // failure
   if(s_Address == INADDR_NONE)
   {
      struct hostent * p_HostEnt;

      // try to get address by the name
      p_HostEnt = gethostbyname(qs_ComputerName);

      if(p_HostEnt != NULL)
      {
         char * cp_IP;
         cp_IP = p_HostEnt->h_addr_list[0];

         // we got the address
         s_Address =  * ((in_addr_t *) cp_IP);
      }
   }

   // return the address
   return s_Address;

} // GetAddress()

//===========================================================================
//
//===========================================================================


void HdClient::CloseConnection ()
{
   close(ClientSocket);

} // CloseConnection();

//===========================================================================
//
//===========================================================================


void HdClient::WaitForTermination ()
{
   // wait for termination
   pthread_join(p_SocketThread,NULL);
   pthread_join(p_PlayerThread,NULL);

} // WaitForTermination

//===========================================================================
//
//===========================================================================

void HdClient::ReinitParameter ()
{
   // set the valid device
   m_pc_Mixer-> SetMixerDevice(m_qs_MixerDevice);

   // call the mixerdevice object
   m_pc_Mixer-> SetOutputMixer(m_i_MixerOutputDevice,
                                m_i_OutputLevel,
                                m_i_OutputLevelMaster);    


   pc_AudioPlayer->SetDevice(m_qs_SoundDevice);
   pc_AudioPlayer->SetSpeed(44100);
} // ReinitParameter()

//===========================================================================
//
//===========================================================================






