// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// connection camera to basisdata


#include <stdlib.h>

#include <qapplication.h>
#include <qcursor.h>


#include "basedef.h"
#include "hdclient_basis.h"
#include "hdclient.h"
#include "basisdaten.h"

/*--------------------------------------------------------------------------*/
//
// constant names for the base settings variables
//
#define BASISDATENNAME              ".KHdClientrc"
#define BASISDATAOUTPUTDEVICE       "OutputDevice"
#define BASISDATAOUTPUTLEVEL        "OutputLevel"
#define BASISDATAOUTPUTLEVELMASTER  "OutputLevelMaster"
#define BASISDATASOUNDDEVICE        "SoundDevice"
#define BASISDATAMIXERDEVICE        "MixerDevice"
#define BASISDATAIPADDRESS	    "IPAddress"
#define BASISDATAIPPORT		    "IPPort"


/*--------------------------------------------------------------------------*/

static char *BasicDataName()
{
   static char exportName[500];

   char        *envReturn;
   
   
   envReturn =getenv("HOME");

   if(envReturn ==NULL)
   {
      strcpy(exportName,BASISDATENNAME);
   }
   else
   {
      strcpy(exportName,envReturn);
      strcat(exportName,"/");
      strcat(exportName,BASISDATENNAME);
   }

   return exportName;

} // BasicDataName()

//-------------------------------------------------------------------

void BasicDataRead(HdClient *p_Application)
{
   BasisDatenType *p_Basic;
   char            sz_InputDevice [500];
   char            sz_Filename    [1500];
   bool            gelungen;
   int             i_Value;
   
   p_Basic =  p_Application->m_BasicData;
   if(p_Basic == NULL)
   {
      return;
   }

   gelungen = p_Basic->BasisDatenLesen(BasicDataName());
   if(!gelungen)
      return;
      
   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAOUTPUTDEVICE,
   		sz_InputDevice);
   if(gelungen)
   {
      if(strcmp(sz_InputDevice,"Line") ==0)
      {
         p_Application->m_i_MixerOutputDevice = SOUND_MIXER_LINE;
      }
      else
      {
         p_Application->m_i_MixerOutputDevice = SOUND_MIXER_LINE;
      }
   }
   else
   {
      p_Application->m_i_MixerOutputDevice = SOUND_MIXER_LINE;
   }

   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAIPADDRESS,
   		sz_InputDevice);
   if(gelungen)
   {
      p_Application->m_qs_IPAddress = sz_InputDevice;
   }
   else
   {
      p_Application->m_qs_IPAddress = "localhost";
   }



   p_Application->m_i_IPPort = DEFAULT_SERVER_PORT;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAIPPORT,
   		&p_Application->m_i_IPPort);
   if(   p_Application->m_i_IPPort <  0 
      || p_Application->m_i_IPPort >= 0x8000
     )
   {
      p_Application->m_i_IPPort = DEFAULT_SERVER_PORT;
   }

   p_Application->m_i_OutputLevel = 70;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAOUTPUTLEVEL,
   		&p_Application->m_i_OutputLevel);
   if(   p_Application->m_i_OutputLevel < 0
      || p_Application->m_i_OutputLevel > 100 
     )
   {
      p_Application->m_i_OutputLevel = 70;
   }
   p_Application->m_i_OutputLevelMaster = 70;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAOUTPUTLEVELMASTER,
   		&p_Application->m_i_OutputLevelMaster);
   if(   p_Application->m_i_OutputLevelMaster < 0
      || p_Application->m_i_OutputLevelMaster > 100 
     )
   {
      p_Application->m_i_OutputLevelMaster = 70;
   }

   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATASOUNDDEVICE,
   		sz_Filename );
   if(gelungen)
   {
      p_Application->m_qs_SoundDevice = sz_Filename;
   }

   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAMIXERDEVICE,
   		sz_Filename );
   if(gelungen)
   {
      p_Application->m_qs_MixerDevice = sz_Filename;
   }

   		

} // BasicDataRead

//-------------------------------------------------------------------

void BasicDataWrite(HdClient  *p_Application)
{
   BasisDatenType *p_Basic;
   bool            gelungen;
   char            sz_InputDevice [500];
   char            sz_Filename[1500];
   
   p_Basic =  p_Application->m_BasicData;
   switch(p_Application->m_i_MixerOutputDevice)
   {
      case SOUND_MIXER_LINE:
         strcpy(sz_InputDevice,"Line");
         break;
      default:
         strcpy(sz_InputDevice,"Line");
         break;
   }

   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAOUTPUTDEVICE,
   		sz_InputDevice);



   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAOUTPUTLEVEL,
   		p_Application-> m_i_OutputLevel);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAOUTPUTLEVELMASTER,
   		p_Application-> m_i_OutputLevelMaster);

   strcpy(sz_Filename,p_Application->m_qs_IPAddress);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAIPADDRESS,
   		sz_Filename);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAIPPORT,
   		p_Application-> m_i_IPPort);


   strcpy(sz_Filename,p_Application->m_qs_SoundDevice);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATASOUNDDEVICE,
   		sz_Filename);

   strcpy(sz_Filename,p_Application->m_qs_MixerDevice);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAMIXERDEVICE,
   		sz_Filename);

   gelungen = p_Basic->BasisDatenSchreiben(BasicDataName());
  
} // BasicDataWrite()




