// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// main-modules for khdrecord



#include <qapplication.h>
#include <qcursor.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>

#include <qmotifstyle.h>
#include <qwindowsstyle.h>
#include <qplatinumstyle.h>


#include <stdio.h>
#include <stdlib.h>

#include "hdclient.h"
#include "commandline.h"
#include "maindialog.h"
#include "aboutbox.h"

/*---------------------------------------------------------------*/

//===========================================================================
//
//===========================================================================

void PrintHelp()
{
   printf("KHdClient Version %3.1f  - Description of Parameters \n",KHDCLIENT_VERSION);
   printf("-command\n");
   printf("\tSwitch to command line mode\n");
   printf("\tMust be the first parameter\n");
   printf("-sounddevice=<device>\n");
   printf("\tSound device like /dev/dsp\n");
   printf("-mixerdevice=<device>\n");
   printf("\tMixer device like /dev/mixer\n");
   printf("-recorderfile=<filename>\n");
   printf("\tFile containig record lists\n");
   printf("-level=<number>\n");
   printf("\tLevel of the output channel\n");
   printf("-master=<number>\n");
   printf("\tMaster level of the soundcard\n");
   printf("-serverport=<number>\n");
   printf("\tPort on which the server waits for the client\n");
   printf("-server=<IP-Address>\n");
   printf("\tName or IP address of the server\n");
} // PrintHelp


//===========================================================================
//
//===========================================================================


int main(int argc, char *argv[])
{
   CommandLineParameter   *pc_CommandLineParameter;
   QApplication           *pc_Application;
   HdClient               *pc_Player;
   MainDialog             *pc_MainDialog;

   bool                   b_CommandLineVersion;

   b_CommandLineVersion =false;
   if(argc >1)
   {
      if(strcmp(argv[1],"-command") ==0)
      {
	 fprintf(stderr,"KHdClient command line version\n");
         b_CommandLineVersion    = true;
         pc_CommandLineParameter = new CommandLineParameter(argc,argv);
         if(! pc_CommandLineParameter->m_b_ParseOK)
         {
            exit(3);
         }
         pc_Application = NULL;
      }
      if(strcmp(argv[1],"-help") ==0)
      {
         PrintHelp();
         exit(4);
      }
   }

   // not command line version
   if( ! b_CommandLineVersion )
   {
      // then create a GUI application
      pc_Application = new QApplication(argc,argv);
   }
   else
   {
      // no GUI
      pc_Application = NULL;
   }




   if( ! b_CommandLineVersion )
   {
      // create recording main class
      pc_Player = new HdClient((QWidget*) pc_Application,NULL);

      pc_MainDialog = new MainDialog(0,pc_Application,pc_Player);
      pc_Player->SetMainDialog(pc_MainDialog);
   
      // show the window
      pc_MainDialog->show();

      // execte main window   
      return pc_Application->exec();

   }
   else // command line version
   {
      // create recording main class
      pc_Player = new HdClient(0,NULL);

      pc_Player->m_ProgramState.b_CommandLineMode = true;

      // execute recording from the command line parameters
      pc_Player->m_i_OutputLevel       = pc_CommandLineParameter ->m_i_OutputLevel;
      pc_Player->m_i_OutputLevelMaster = pc_CommandLineParameter ->m_i_OutputLevelMaster;
      pc_Player->m_i_MixerOutputDevice = pc_CommandLineParameter ->m_i_OutputDevice;
      pc_Player->m_qs_SoundDevice      = pc_CommandLineParameter ->m_qs_SoundDevice;
      pc_Player->m_qs_MixerDevice      = pc_CommandLineParameter ->m_qs_MixerDevice;

      pc_Player->m_qs_IPAddress        = pc_CommandLineParameter ->m_qs_ServerAddress;
      pc_Player->m_i_IPPort            = pc_CommandLineParameter ->m_i_ServerPort;

      pc_Player->ReinitParameter();

      pc_Player->DoStart();

      // endless loop
      pc_Player->WaitForTermination();
   } // else command line version


   
} // main()
















