// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancements in the original file.
//
// main hdclient application window class

// #define _DEBUG

#include <qapplication.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>


#include "maindialog.h"
#include "aboutbox.h"
#include "hdclient.h"
#include "settingsbox.h"


//===========================================================================
// constructor for the recording program
//===========================================================================

MainDialog::MainDialog(QWidget *parent,
               QApplication *mainApplication,
               HdClient *pc_Player)
   		: QMainWindow(parent,
   		              "KHdClient",
   		              WType_TopLevel )
{
   QBoxLayout  *TopLayout;
   QWidget     *TopWidget;

   m_pc_Player = pc_Player;

   m_pc_MainApplication = mainApplication;

   // set the window title
   {
      char cs_TitleString[100];
      sprintf(cs_TitleString,"KHdClient Version %.1f",KHDCLIENT_VERSION);
      setCaption(cs_TitleString);
   }

   // create the menu   
   m_FileMenu = new QPopupMenu( this );
   m_FileMenu->insertItem("About KHdClient", this, SLOT( DoAboutBox() ) );
   m_FileMenu->insertItem("Settings", this, SLOT( DoSettingsBox() ) );
   m_FileMenu->insertItem("Exit" , this, SLOT( Beenden() ) );

   // add menus to menubars
   menuBar()->insertItem("File"                 , m_FileMenu );


   
   // topwidgets erzeugen
   TopWidget = new QWidget(this);
   setCentralWidget(TopWidget);
   
  
      
   TopLayout     = new QVBoxLayout( TopWidget,1);

   // block for the sample level
   {
      QGridLayout     * SliderLayout;
      QWidget         * OutputLevelWidget;
      QLabel          * DescriptionLabel;
      QLabel          * DescriptionMasterLabel;
      char             LevelString[30];
      

      OutputLevelWidget = new QWidget(TopWidget);
      TopLayout->addWidget(OutputLevelWidget);
     
      SliderLayout = new QGridLayout( OutputLevelWidget,2,3);
	
      DescriptionLabel       = new QLabel("Output Level: ",
                                           OutputLevelWidget); 
      DescriptionMasterLabel = new QLabel("Master Level: ",
                                           OutputLevelWidget); 
      
      OutputLevelSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep, 
                                      50, 		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      OutputLevelWidget);
                                      
      OutputLevelMasterSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep, 
                                      50, 		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      OutputLevelWidget);

       
      sprintf(LevelString,"%3d",100);
      OutputLevelLabel       = new QLabel(LevelString,OutputLevelWidget);  
      OutputLevelMasterLabel = new QLabel(LevelString,OutputLevelWidget);  


      SliderLayout->addWidget(DescriptionLabel,0,0);
      SliderLayout->addWidget(DescriptionMasterLabel,1,0);
      SliderLayout->addWidget(OutputLevelSlider,0,1);
      SliderLayout->addWidget(OutputLevelMasterSlider,1,1);
      SliderLayout->addWidget(OutputLevelLabel ,0,2);
      SliderLayout->addWidget(OutputLevelMasterLabel ,1,2);

      
      connect( OutputLevelSlider, 
               SIGNAL(valueChanged(int)), 
               SLOT(DoSliderValueChanged(int)) );
      connect( OutputLevelMasterSlider, 
               SIGNAL(valueChanged(int)), 
               SLOT(DoMasterSliderValueChanged(int)) );

   } // block for sample level

   // block fuer connect/stop button   
   {
      QGridLayout *ButtonsLayout;
      QWidget     *ButtonWidget;
      
      ButtonWidget = new QWidget(TopWidget);
      TopLayout->addWidget(ButtonWidget);
      
      ButtonsLayout = new QGridLayout( ButtonWidget, 1, 2,2);


      ConnectButton  = new QPushButton("Connect",ButtonWidget);
      StopButton   = new QPushButton("Stop",ButtonWidget);
      ButtonsLayout->addWidget (ConnectButton,0,0);
      ButtonsLayout->addWidget (StopButton,0,1);

   } // block for start stop 

   // connect the buttons
   connect(ConnectButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoConnectButton()) );

   // connect the stop button
   connect(StopButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoStopButton()) );

   {
      QGridLayout *LineLayout;
      QWidget     *LineWidget;

      LineWidget = new QWidget(TopWidget);
      LineLayout = new QGridLayout( LineWidget, 1, 2,1);

      m_pc_SampleArea = new SampleArea(LineWidget);
      StatusLabel = new QLabel("",LineWidget);

      LineLayout->addWidget (m_pc_SampleArea,0,0);
      LineLayout->addWidget (StatusLabel,0,1);

      TopLayout->addWidget(LineWidget);
   }


} // MainDialog()

//===========================================================================
// update the time display
//===========================================================================

MainDialog::~MainDialog()
{
   
} // destructor



//===========================================================================
// display the about box
//===========================================================================

void MainDialog::DoAboutBox()
{
   AboutBox * aboutBox;

   aboutBox = new AboutBox(this);
   
   aboutBox->exec();
   
   delete aboutBox;
} // DoAboutBox()



//===========================================================================
// eventfunction for the start button
//===========================================================================


void MainDialog::DoConnectButton()
{
   m_pc_Player->DoStart();

} // DoStartButton()

//===========================================================================
// eventfunction for the stop button
//===========================================================================


void MainDialog::DoStopButton()
{

   m_pc_Player->DoStop();

   
} // DoStopButton()


//===========================================================================
//
//===========================================================================

void MainDialog::TurnGraphics()
{
   m_pc_MainApplication->lock();
   m_pc_SampleArea->SetNextState();
   FlushDisplay();
   m_pc_MainApplication->unlock();
} // TurnGraphics()



//===========================================================================
//
//===========================================================================


void MainDialog::FlushDisplay()
{
   // not needed anymore
   // flush all X events
   //- m_pc_MainApplication->flushX();
   // for QT2 I have also to use this call
   //- m_pc_MainApplication->sendPostedEvents();
   //- m_pc_MainApplication->processEvents();

}  //  FlushDisplay()


//===========================================================================
//
//===========================================================================

void MainDialog::SetSensitivity(ProgramState c_ProgramState)
{
   
   if(c_ProgramState.b_PlayStarted)
   {
      // disable all elements, except the stop button
      ConnectButton       -> setEnabled(false);
      StopButton          -> setEnabled(true);
      
   } 
   else  // stopped
   {
      // disable all elements, except the stop button
      ConnectButton       -> setEnabled(true);
      StopButton          -> setEnabled(false);
      
      // clean up the sample area
      m_pc_SampleArea->Initialize();
   } 

} // SetSensitivity()


//===========================================================================
//
//===========================================================================

void MainDialog::SetOutputLevel(int i_OutputLevel)
{
   OutputLevelSlider -> setValue(i_OutputLevel);
} // SetOutputLevel()

//===========================================================================
//
//===========================================================================

void MainDialog::SetOutputLevelMaster(int i_OutputLevelMaster)
{
   OutputLevelMasterSlider -> setValue(i_OutputLevelMaster);
} // SetOutputLevelMaster()


//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoSliderValueChanged(int i_SliderValue)
{
   char LevelString[30];
   
   // write it out
   sprintf(LevelString,"%3d",i_SliderValue);

   OutputLevelLabel->setText(LevelString);  

   m_pc_Player->DoSliderValueChanged(i_SliderValue);

} // DoSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoMasterSliderValueChanged(int i_SliderValue)
{
   char LevelString[30];
   
   // write it out
   sprintf(LevelString,"%3d",i_SliderValue);
   OutputLevelMasterLabel->setText(LevelString);  

   m_pc_Player->DoMasterSliderValueChanged(i_SliderValue);
   
} // DoSliderValueChanged()


//===========================================================================
// terminate
//===========================================================================


void MainDialog::Beenden()
{
   m_pc_Player->Beenden();
} // Beenden()



//===========================================================================
// catch the close event from the main window x-corner
//===========================================================================

void MainDialog::closeEvent(QCloseEvent *)
{
   Beenden();
} // closeEvent


//===========================================================================
//
//===========================================================================

void MainDialog::SetStatusLabel(QString qs_StatusString)
{
   m_pc_MainApplication->lock();
   StatusLabel->setText(qs_StatusString);
   m_pc_MainApplication->unlock();
} // SetStatusLabel()



//=============================================================================
//
//=============================================================================

void MainDialog::DoSettingsBox()
{
   SettingsBox * settingsBox;
 
   settingsBox = new SettingsBox(this,m_pc_Player);
   
   settingsBox->exec();
  
   // check again, whether the devices are available
   // m_pc_Player->CheckDevices();   

   delete settingsBox;
} // DoSettingsBox()

//=============================================================================
//
//=============================================================================


