// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// sample level output widget

#include <stdio.h>


#include <qpainter.h>

#include "samplearea.h"


//=========================================================================
// constructor
//=========================================================================

SampleArea::SampleArea(QWidget *parent)
		: QWidget(parent,"SampleArea")
{
   int i;

   // set to exact 16x16 dimension of the area
   setMinimumSize(16,16);
   setMaximumSize(16,16);
   i_Width = width();
   i_Height = height();
   
   // allocate both colors
   pc_WhiteColor = new QColor(0xff,0xff,0xff);
   pc_BlackColor = new QColor(0x00,0x00,0x00);
   
   // allocate a brush
   pc_Brush      = new QBrush(*pc_WhiteColor);
   
   
   m_i_State = -1;   
}


//=========================================================================
//
//=========================================================================

SampleArea::~SampleArea()
{
   // delete the brush
   pc_Brush      = new QBrush(*pc_WhiteColor);

   // delete the colors
   delete pc_WhiteColor ;
   delete pc_BlackColor ;
   
} // ~SampleArea()

//=========================================================================
// repaint the area.
// if theEvent is NULL, the function was called from the programm
// if theEvent is valid, it is called by the X-System
//=========================================================================
		
void  SampleArea::paintEvent(QPaintEvent *theEvent)
{
   QPainter painter(this);
   int      i;


   switch(m_i_State)
   {   
   case 0:
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);
      break;
   case 1:
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

      pc_Brush->setColor(*pc_BlackColor);
      painter.fillRect(0,0,i_Width,i_Height/2,*pc_Brush);

      break;
   case 2:
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

      pc_Brush->setColor(*pc_BlackColor);
      painter.fillRect(i_Width/2,0,i_Width,i_Height,*pc_Brush);

      break;
   case 3:
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

      pc_Brush->setColor(*pc_BlackColor);
      painter.fillRect(0,i_Height/2,i_Width,i_Height,*pc_Brush);

      break;
   case 4:
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

      pc_Brush->setColor(*pc_BlackColor);
      painter.fillRect(0,0,i_Width/2,i_Height,*pc_Brush);

      break;
   default:
      break;
   }

   // flush for a complete redraw
   painter.flush();

} // paintEvent()

//=========================================================================
//
//=========================================================================


void  SampleArea::resizeEvent(QResizeEvent *theEvent)
{
   i_Width  = width();
   i_Height = height();
}


//=========================================================================
// set a new array of sample values
//=========================================================================

void  SampleArea::SetNextState()
{
   m_i_State +=1;
   if(m_i_State >4)
   {
      m_i_State =1;
   }
   paintEvent(NULL);
} // SetValues()

//=========================================================================
//
//=========================================================================

void  SampleArea::Initialize()
{
   m_i_State =0;

} // Inititalize()



