// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#include <stdio.h>
#include <unistd.h>

#include "saveblock.h"

//===========================================================================
// constructor for one saveblock
//===========================================================================

SaveBlock::SaveBlock(size_t size)
{
   if(size >0)
   {
      Buffer = new unsigned char[size];
      if(Buffer != NULL)
      {
         BufferSize = size;
      }
      else
      {
         BufferSize =0;
      }
      BufferFilled =0;
      BufferSavePosition =NULL;
   
   }
   else // zero size, should not happen
   {
      Buffer             = NULL;
      BufferSize         =0;
      BufferFilled       =0;
      BufferSavePosition =NULL;
   }

   // reset the trigger for the last block
   b_FinalBlock = false;
   
   m_i_Level = 5000;
}

//===========================================================================
// delete the saveblocks
//===========================================================================

SaveBlock::~SaveBlock()
{
   if(Buffer != NULL)
   {
      delete[] Buffer;
      Buffer = NULL;
   }
}

//===========================================================================
// write on block into a file
//===========================================================================


int SaveBlock::WriteBlock(int OutFileHandle)
{

   if( OutFileHandle <=0 )
   {
      return 0;
   }
   
   // write the data
   write(OutFileHandle,Buffer,BufferFilled);
   
   // remember: this block is empty now
   BufferFilled=0;
   
   
   return 0;

} // WriteBlock()


//===========================================================================
// check, whether it is the final block
//===========================================================================

bool SaveBlock::IsFinalBlock()
{
   return b_FinalBlock;
}

//===========================================================================
// set the final block flag
//===========================================================================

void SaveBlock::SetFinalBlock()
{
   b_FinalBlock = true;
} // SetFinalBlock()



//###########################################################################
// now create an array of safeblocks
//###########################################################################


//===========================================================================
// constructor

//===========================================================================
SaveBlockArray::SaveBlockArray()
{
   int i;
   
   // allocate the buffers
   for( i=0; i < SAVEBLOCKARRAYSIZE; i++)
   {
      // create a new block, size hardcoded 196K
      pc_SaveBlocks[i]= new SaveBlock(0x30000);
   }
   
   
   // init the variables
   
   NextRecordBlock = 0;
   NextWriteBlock  = 0;
   
   // create the semaphore for sync of record/write
   sem_init(&BufferSemaphore,
            0,                  // process share
            0                   // initital value
            );
   
} //SaveBlockArray()

//===========================================================================
// destructor
//===========================================================================


SaveBlockArray::~SaveBlockArray()
{
   int i;
   
   // free all blocks
   for(i=0; i<SAVEBLOCKARRAYSIZE; i++)
   {
      if( pc_SaveBlocks[i] != NULL)
      {
         delete pc_SaveBlocks[i];
         pc_SaveBlocks[i] = NULL;
      }
   }
   
   // now delete the semaphore
   sem_destroy(&BufferSemaphore);
   
} // ~SaveBlockArray()

//===========================================================================
// get next record block
// assume, there is always one, so no semaphore is used
//===========================================================================


SaveBlock *SaveBlockArray::GetNextRecordBlock()
{
 
   SaveBlock *pc_NextBlock;
 
   pc_NextBlock = pc_SaveBlocks[NextRecordBlock];
   
   NextRecordBlock += 1;
   if(NextRecordBlock >= SAVEBLOCKARRAYSIZE)
   {
      NextRecordBlock =0;
   }
   
   return pc_NextBlock;
}

//===========================================================================
// release a block for the writing
//===========================================================================


void SaveBlockArray::ReleaseRecordBlock()
{
   // simply increment the semaphore
   sem_post(&BufferSemaphore);

#ifdef _PETER
{ 
int sem_value;
sem_getvalue(&BufferSemaphore, &sem_value);
fprintf(stderr,"sem =%d\n",sem_value);
}
#endif

}

//===========================================================================
// get a block for writing.
// because the can be none available, wait with the semaphore for a block
//===========================================================================

SaveBlock *SaveBlockArray::GetNextWriteBlock()
{

 
   SaveBlock *pc_NextBlock;

   // wait for the block
   sem_wait(&BufferSemaphore);
   
   // get block
   pc_NextBlock = pc_SaveBlocks[NextWriteBlock];
   
   NextWriteBlock += 1;
   if(NextWriteBlock >= SAVEBLOCKARRAYSIZE)
   {
      NextWriteBlock =0;
   }
   
   
   return pc_NextBlock;
   
} // GetNextWriteBlock()

//===========================================================================
//
//===========================================================================


void SaveBlockArray::Initialize()
{
   // init the variables
   
   NextRecordBlock = 0;
   NextWriteBlock  = 0;
   
   // now delete the semaphore
   sem_destroy(&BufferSemaphore);

   // create the semaphore for sync of record/write
   sem_init(&BufferSemaphore,
            0,                  // process share
            0                   // initital value
            );

} // Initialize()


